﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ACME_2000_car
{
	public abstract class Automobile : IVehicleModel
	{
		private ArrayList aList = new ArrayList();
		public Automobile(int paramMaxSpeed, int paramMaxTurnSpeed, 
			int paramMaxReverseSpeed, string paramName)
		{
			Direction = AbsoluteDirection.North;
			MaxSpeed = paramMaxSpeed;
			MaxTurnSpeed = paramMaxTurnSpeed;
			MaxReverseSpeed = paramMaxReverseSpeed;
			Name = paramName;
		}
		
		#region "IVehicleModel Members"
		public string Name { get; set; }
		public int Speed { get; set; }
		public int MaxSpeed { get; private set; }
		public int MaxTurnSpeed { get; private set; }
		public int MaxReverseSpeed { get; private set; }
		public AbsoluteDirection Direction { get; set; }
		public void AddObserver(IVehicleView paramView)
		{
			aList.Add(paramView);
		}
		public void RemoveObserver(IVehicleView paramView)
		{
			aList.Remove(paramView);
		}
		public void NotifyObservers()
		{
			foreach (IVehicleView view in aList)
			{
				view.Update(this);
			}
		}
	  public void Turn(RelativeDirection paramDirection)
		{
			AbsoluteDirection newDirection;
			switch (paramDirection)
			{
				case RelativeDirection.Right:
					newDirection = (AbsoluteDirection)((int)(Direction + 1) % 4);
					break;
				case RelativeDirection.Left:
					newDirection = (AbsoluteDirection)((int)(Direction + 3) % 4);
					break;
				case RelativeDirection.Back:
					newDirection = (AbsoluteDirection)((int)(Direction + 2) % 4);
					break;
				default:
					newDirection = AbsoluteDirection.North;
					break;
			}
			Direction = newDirection;
			NotifyObservers();
		}
		public void Accelerate(int paramAmount)
		{
			Speed += paramAmount;
			if (Speed >= MaxSpeed) Speed = MaxSpeed;
			NotifyObservers();
		}
		public void Decelerate(int paramAmount)
		{
			Speed -= paramAmount;
			if (Speed <= MaxReverseSpeed) Speed = MaxReverseSpeed;
			NotifyObservers();
		}
		#endregion
	}
}
