﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Szoveg.Annotations;

namespace Szoveg
{
	public class ViewModel:IViewModel,INotifyPropertyChanged
	{
		/// <summary>
		/// Modell objektum referenciájának tárolásásra szolgáló adattag.
		/// </summary>
		private IModel model;
		/// <summary>
		/// Állománynév tárolására szolgáló adattag.
		/// </summary>
		string ÁllományNév = "valami.txt";
		public ViewModel(IView wnd)
		{
			// Parancscsatoló objektum létrehozása a mentés funkcióhoz.
			var cbMentés =
				new CommandBinding(ApplicationCommands.Save);
			// Eseménykezelő metódus parancshoz rendelése
			cbMentés.Executed += cbMentés_Executed;

			// Parancscsatoló objektum létrehozása a megnyitás funkcióhoz.
			var cbMegnyitás =
				new CommandBinding(ApplicationCommands.Open);
			// Eseménykezelő metódus parancshoz rendelése
			cbMegnyitás.Executed += cbMegnyitás_Executed;

			// Parancscsatoló objektum létrehozása az új dokumentum funkcióhoz.
			var cbÚj = new CommandBinding(
					ApplicationCommands.New);
			// Eseménykezelő metódus parancshoz rendelése
			cbÚj.Executed += cbÚj_Executed;

			var cbKilépés = new CommandBinding(
				ApplicationCommands.Close);
			cbKilépés.Executed += cbKilépés_Executed;

			// Parancscsatoló objektumok felvétele az ablak parancscsatoló listájába.
			wnd.CommandBindings.Add(cbMentés);
			wnd.CommandBindings.Add(cbMegnyitás);
			wnd.CommandBindings.Add(cbÚj);
			wnd.CommandBindings.Add(cbKilépés);

			// Az adatkötéshez megadjuk adatforrásként az aktuális ViewModel típusú objektumot a View objektum számára. 
			// Ezáltal a felület TextBox komponensének Text tulajdonsága kötésbe kerül a Szöveg tulajdonsággal.
			wnd.DataContext = this;

			// Modell objektum létrehozása és a ViewModel objektum referenciájának átadása.
			model=new Model(this);
		}

		/// <summary>
		/// Szerepe csak annyi, hogy hozzáférést ad a modellben tárolt adathoz. Az adatkötés a ViewModel ezen tulajdonságához történik.
		/// </summary>
		public string Szöveg
		{
			get { return model.Szöveg; }
			set { model.Szöveg = value; OnPropertyChanged("Szöveg"); }
		}
		public void cbÚj_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			// Ha megváltozott a modellben tárolt adat a legutóbbi mentés óta, akkor felkínálja a mentés lehetőségét a felhasználónak.
			VáltozásEllenőrzésMentés();
			// Szövegmező és modelladat tartalmának ürítése.
			Szöveg="";
			model.VoltVáltozás = false;
		}
		public void cbMegnyitás_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			// Ha megváltozott a modellben tárolt adat a legutóbbi mentés óta, akkor felkínálja a mentés lehetőségét a felhasználónak.
			VáltozásEllenőrzésMentés();
			Betöltés();
			MessageBox.Show("Állomány betöltve!");
		}
		public void cbMentés_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			// Az adatok lemetése egy állományba.
			Mentés();
			MessageBox.Show("Állomány mentve!");
		}
		public void cbKilépés_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			// Ha megváltozott a modellben tárolt adat a legutóbbi mentés óta, akkor felkínálja a mentés lehetőségét a felhasználónak.
			VáltozásEllenőrzésMentés();
			// Alkalmazás leállítása
			Application.Current.Shutdown();
		}
		/// <summary>
		/// Ellenőrzi, hogy módosult-e a modellben tárolt adat alegutóbbi mentés óta. Ha igen akkor felkínálja a mentés lehetőségét a felhasználónak.
		/// </summary>
		private void VáltozásEllenőrzésMentés()
		{
			// Ha megváltozott a modellben tárolt adat értéke a legutóbbi mentés óta.
			if (!model.VoltVáltozás) return;
			// Akar-e menteni a felhasználó?
			var res = MessageBox.Show("Akarsz menteni?", "kérdés",
				MessageBoxButton.YesNo, MessageBoxImage.Question);
			if (res != MessageBoxResult.Yes) return;
			// Az adatok lemetése egy állományba.
			Mentés();
			MessageBox.Show("Állomány mentve!");
		}

		/// <summary>
		/// Betölti az adatokat egy állományból.
		/// </summary>
		private void Betöltés()
		{
			// Beolvasást végző objektum létrehozása.
			var sr = new StreamReader(ÁllományNév);
			// Az állomány teljes tartalmának beolvasása.
			Szöveg = sr.ReadToEnd();
			// Állomány lezárása.
			sr.Close();
			// Modell állapotának frissítése.
			model.VoltVáltozás = false;
		}
		/// <summary>
		/// Lementi az adatokat egy állományba.
		/// </summary>
		private void Mentés()
		{
			// Mentést végző objektum létrehozása, használata, lezárása.
			using (var sw = new StreamWriter(ÁllományNév))
			{
				// Szövegmező tartalmának mentése.
				sw.Write(Szöveg);
			}
			// Modell állapotának frissítése.
			model.VoltVáltozás = false;
		}

		#region INotifyPropertyChanged
		public event PropertyChangedEventHandler PropertyChanged;

		[NotifyPropertyChangedInvocator]
		protected virtual void OnPropertyChanged([CallerMemberName] string propertyName = null)
		{
			if (PropertyChanged != null)
				PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
		}
		#endregion
	}
}
